@@ -14,6 +14,7 @@
 #include "base/exception.hpp"
 #include "base/convert.hpp"
 #include "base/statsfunction.hpp"
+#include <chrono>
 
 using namespace icinga;
 
@@ -44,10 +45,16 @@ void CheckerComponent::StatsFunc(const Dictionary::Ptr& status, const Array::Ptr
 
 void CheckerComponent::OnConfigLoaded()
 {
-	ConfigObject::OnActiveChanged.connect(std::bind(&CheckerComponent::ObjectHandler, this, _1));
-	ConfigObject::OnPausedChanged.connect(std::bind(&CheckerComponent::ObjectHandler, this, _1));
+	ConfigObject::OnActiveChanged.connect([this](const ConfigObject::Ptr& object, const Value&) {
+		ObjectHandler(object);
+	});
+	ConfigObject::OnPausedChanged.connect([this](const ConfigObject::Ptr& object, const Value&) {
+		ObjectHandler(object);
+	});
 
-	Checkable::OnNextCheckChanged.connect(std::bind(&CheckerComponent::NextCheckChangedHandler, this, _1));
+	Checkable::OnNextCheckChanged.connect([this](const Checkable::Ptr& checkable, const Value&) {
+		NextCheckChangedHandler(checkable);
+	});
 }
 
 void CheckerComponent::Start(bool runtimeCreated)
@@ -58,18 +65,18 @@ void CheckerComponent::Start(bool runtimeCreated)
 		<< "'" << GetName() << "' started.";
 
 
-	m_Thread = std::thread(std::bind(&CheckerComponent::CheckThreadProc, this));
+	m_Thread = std::thread([this]() { CheckThreadProc(); });
 
 	m_ResultTimer = new Timer();
 	m_ResultTimer->SetInterval(5);
-	m_ResultTimer->OnTimerExpired.connect(std::bind(&CheckerComponent::ResultTimerHandler, this));
+	m_ResultTimer->OnTimerExpired.connect([this](const Timer * const&) { ResultTimerHandler(); });
 	m_ResultTimer->Start();
 }
 
 void CheckerComponent::Stop(bool runtimeRemoved)
 {
 	{
-		boost::mutex::scoped_lock lock(m_Mutex);
+		std::unique_lock<std::mutex> lock(m_Mutex);
 		m_Stopped = true;
 		m_CV.notify_all();
 	}
@@ -88,7 +95,7 @@ void CheckerComponent::CheckThreadProc()
 	Utility::SetThreadName("Check Scheduler");
 	IcingaApplication::Ptr icingaApp = IcingaApplication::GetInstance();
 
-	boost::mutex::scoped_lock lock(m_Mutex);
+	std::unique_lock<std::mutex> lock(m_Mutex);
 
 	for (;;) {
 		typedef boost::multi_index::nth_index<CheckableSet, 1>::type CheckTimeView;
@@ -116,7 +123,7 @@ void CheckerComponent::CheckThreadProc()
 
 		if (wait > 0) {
 			/* Wait for the next check. */
-			m_CV.timed_wait(lock, boost::posix_time::milliseconds(long(wait * 1000)));
+			m_CV.wait_for(lock, std::chrono::duration<double>(wait));
 
 			continue;
 		}
@@ -127,12 +134,15 @@ void CheckerComponent::CheckThreadProc()
 
 		bool forced = checkable->GetForceNextCheck();
 		bool check = true;
+		bool notifyNextCheck = false;
 
 		if (!forced) {
 			if (!checkable->IsReachable(DependencyCheckExecution)) {
 				Log(LogNotice, "CheckerComponent")
 					<< "Skipping check for object '" << checkable->GetName() << "': Dependency failed.";
+
 				check = false;
+				notifyNextCheck = true;
 			}
 
 			Host::Ptr host;
@@ -156,7 +166,9 @@ void CheckerComponent::CheckThreadProc()
 				Log(LogNotice, "CheckerComponent")
 					<< "Skipping check for object '" << checkable->GetName()
 					<< "': not in check period '" << tp->GetName() << "'";
+
 				check = false;
+				notifyNextCheck = true;
 			}
 		}
 
@@ -170,6 +182,11 @@ void CheckerComponent::CheckThreadProc()
 
 			checkable->UpdateNextCheck();
 
+			if (notifyNextCheck) {
+				// Trigger update event for Icinga DB
+				Checkable::OnNextCheckUpdated(checkable);
+			}
+
 			lock.lock();
 
 			continue;
@@ -198,7 +215,13 @@ void CheckerComponent::CheckThreadProc()
 
 		Checkable::IncreasePendingChecks();
 
-		Utility::QueueAsyncCallback(std::bind(&CheckerComponent::ExecuteCheckHelper, CheckerComponent::Ptr(this), checkable));
+		/*
+		 * Explicitly use CheckerComponent::Ptr to keep the reference counted while the
+		 * callback is active and making it crash safe
+		 */
+		CheckerComponent::Ptr checkComponent(this);
+
+		Utility::QueueAsyncCallback([this, checkComponent, checkable]() { ExecuteCheckHelper(checkable); });
 
 		lock.lock();
 	}
@@ -229,7 +252,7 @@ void CheckerComponent::ExecuteCheckHelper(const Checkable::Ptr& checkable)
 	Checkable::DecreasePendingChecks();
 
 	{
-		boost::mutex::scoped_lock lock(m_Mutex);
+		std::unique_lock<std::mutex> lock(m_Mutex);
 
 		/* remove the object from the list of pending objects; if it's not in the
 		 * list this was a manual (i.e. forced) check and we must not re-add the
@@ -255,7 +278,7 @@ void CheckerComponent::ResultTimerHandler()
 	std::ostringstream msgbuf;
 
 	{
-		boost::mutex::scoped_lock lock(m_Mutex);
+		std::unique_lock<std::mutex> lock(m_Mutex);
 
 		msgbuf << "Pending checkables: " << m_PendingCheckables.size() << "; Idle checkables: " << m_IdleCheckables.size() << "; Checks/s: "
 			<< (CIB::GetActiveHostChecksStatistics(60) + CIB::GetActiveServiceChecksStatistics(60)) / 60.0;
@@ -275,7 +298,7 @@ void CheckerComponent::ObjectHandler(const ConfigObject::Ptr& object)
 	bool same_zone = (!zone || Zone::GetLocalZone() == zone);
 
 	{
-		boost::mutex::scoped_lock lock(m_Mutex);
+		std::unique_lock<std::mutex> lock(m_Mutex);
 
 		if (object->IsActive() && !object->IsPaused() && same_zone) {
 			if (m_PendingCheckables.find(checkable) != m_PendingCheckables.end())
@@ -301,7 +324,7 @@ CheckableScheduleInfo CheckerComponent::GetCheckableScheduleInfo(const Checkable
 
 void CheckerComponent::NextCheckChangedHandler(const Checkable::Ptr& checkable)
 {
-	boost::mutex::scoped_lock lock(m_Mutex);
+	std::unique_lock<std::mutex> lock(m_Mutex);
 
 	/* remove and re-insert the object from the set in order to force an index update */
 	typedef boost::multi_index::nth_index<CheckableSet, 0>::type CheckableView;
@@ -322,14 +345,14 @@ void CheckerComponent::NextCheckChangedHandler(const Checkable::Ptr& checkable)
 
 unsigned long CheckerComponent::GetIdleCheckables()
 {
-	boost::mutex::scoped_lock lock(m_Mutex);
+	std::unique_lock<std::mutex> lock(m_Mutex);
 
 	return m_IdleCheckables.size();
 }
 
 unsigned long CheckerComponent::GetPendingCheckables()
 {
-	boost::mutex::scoped_lock lock(m_Mutex);
+	std::unique_lock<std::mutex> lock(m_Mutex);
 
 	return m_PendingCheckables.size();
 }
